"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.randomBytes = exports.hash = exports.verifyBlob = exports.signBlob = exports.generateKeyPair = void 0;
/*
Copyright 2022 The Sigstore Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
const crypto_1 = require("crypto");
const EC_KEYPAIR_TYPE = 'ec';
const P256_CURVE = 'P-256';
const SHA256_ALGORITHM = 'sha256';
const BASE64_ENCODING = 'base64';
function generateKeyPair() {
    return (0, crypto_1.generateKeyPairSync)(EC_KEYPAIR_TYPE, { namedCurve: P256_CURVE });
}
exports.generateKeyPair = generateKeyPair;
function signBlob(privateKey, blob) {
    const signer = (0, crypto_1.createSign)(SHA256_ALGORITHM);
    signer.update(blob);
    signer.end();
    return signer.sign(privateKey, BASE64_ENCODING);
}
exports.signBlob = signBlob;
function verifyBlob(publicKey, blob, signature) {
    const verifier = (0, crypto_1.createVerify)(SHA256_ALGORITHM);
    verifier.update(blob);
    verifier.end();
    return verifier.verify(publicKey, signature, BASE64_ENCODING);
}
exports.verifyBlob = verifyBlob;
function hash(blob, encoding = 'hex') {
    const hash = (0, crypto_1.createHash)(SHA256_ALGORITHM);
    hash.update(blob);
    return hash.digest(encoding);
}
exports.hash = hash;
function randomBytes(count) {
    return (0, crypto_1.randomBytes)(count);
}
exports.randomBytes = randomBytes;
